package main

import (
	"database/sql"
	"fmt"
	"io"
	"log"
	"net/http"
	"os"
	"strconv"

	"github.com/gin-gonic/gin"
	"github.com/jaswdr/faker"
	_ "github.com/mattn/go-sqlite3"
)

// This is taken from our DB model
type video struct {
	ID       int    `json:"id"`
	Filepath string `json:"filepath"`
}

// videos slice to seed record video data.
var videos []video

func main() {
	// taken from https://github.com/mattn/go-sqlite3/blob/v1.14.22/_example/simple/simple.go
	db := initDb()
	if db == nil {
		log.Fatal("Couldn't create DB")
		return
	}
	defer db.Close()
	defer os.Remove("./videos.db")

	rows, err := db.Query("select * from videos")
	if err != nil {
		log.Fatal(err)
	}
	defer rows.Close()
	for rows.Next() {
		var id int
		var name string
		err = rows.Scan(&id, &name)
		if err != nil {
			log.Fatal(err)
		}
		fmt.Println(id, name)
	}
	err = rows.Err()
	if err != nil {
		log.Fatal(err)
	}

	// Legacy
	faker := faker.New()
	for i := 0; i < 10; i++ {
		videos = append(videos, video{ID: i, Filepath: faker.File().AbsoluteFilePathForUnix(5)})
	}

	router := gin.Default()
	router.SetTrustedProxies(nil)

	router.POST("/upload", ReceiveFile)
	router.GET("/videos", getvideos)
	router.GET("/videos/:id", getvideoByID)
	router.POST("/videos", postvideos)
	router.DELETE("/videos", delvideos)
	router.DELETE("/videos/:id", delvideoByID)
	router.PATCH("/videos/:id", updatevideo)

	router.Run("localhost:8080")
}

func initDb() *sql.DB {
	os.Remove("./videos.db")

	db, err := sql.Open("sqlite3", "./videos.db")
	if err != nil {
		log.Fatal(err)
	}

	sqlStmt := `
	create table videos (id integer not null primary key, filepath text);
	delete from videos;
	`
	_, err = db.Exec(sqlStmt)
	if err != nil {
		log.Printf("%q: %s\n", err, sqlStmt)
		return nil
	}

	faker := faker.New()

	tx, err := db.Begin()
	if err != nil {
		log.Fatal(err)
	}
	stmt, err := tx.Prepare("insert into videos(id, filepath) values(?, ?)")
	if err != nil {
		log.Fatal(err)
	}
	defer stmt.Close()
	for i := 0; i < 100; i++ {
		_, err = stmt.Exec(i, faker.File().AbsoluteFilePathForUnix(5))
		if err != nil {
			log.Fatal(err)
		}
	}
	err = tx.Commit()
	if err != nil {
		log.Fatal(err)
	}

	return db
}

func remove(slice []video, s int) []video {
	return append(slice[:s], slice[s+1:]...)
}

func ReceiveFile(c *gin.Context) {
	// maybe even re-verify mime-type (before or after upload)

	fileName := c.GetHeader("file-name")
	/*
		get data from request
			write append data to file
	*/
	data, err := io.ReadAll(c.Request.Body)
	if err != nil {
		c.IndentedJSON(http.StatusBadRequest, "Unknown Error")
	} else {
		f, err := os.OpenFile(fileName, os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
		if err != nil {
			log.Fatal(err)
		}
		if _, err := f.Write(data); err != nil {
			log.Fatal(err)
		}
		if err := f.Close(); err != nil {
			log.Fatal(err)
		}

		c.IndentedJSON(http.StatusOK, "Received chunk")
	}
}

// getvideos responds with the list of all videos as JSON.
func getvideos(c *gin.Context) {
	c.IndentedJSON(http.StatusOK, videos)
}

// postvideos adds an video from JSON received in the request body.
func postvideos(c *gin.Context) {
	var newvideo video

	// Call BindJSON to bind the received JSON to
	// newvideo.
	if err := c.BindJSON(&newvideo); err != nil {
		c.Status(http.StatusBadRequest)
		return
	}

	for _, a := range videos {
		if a == newvideo {
			c.Status(http.StatusBadRequest)
			return
		}
	}

	// Add the new video to the slice.
	videos = append(videos, newvideo)
	c.IndentedJSON(http.StatusCreated, newvideo)
}

// getvideoByID locates the video whose ID value matches the id
// parameter sent by the client, then returns that video as a response.
func getvideoByID(c *gin.Context) {
	inputId := c.Param("id")

	// Loop through the list of videos, looking for
	// an video whose ID value matches the parameter.
	for _, a := range videos {
		if strconv.Itoa(a.ID) == inputId {
			c.IndentedJSON(http.StatusOK, a)
			return
		}
	}
	c.IndentedJSON(http.StatusNotFound, gin.H{"message": "video not found"})
}

func delvideos(c *gin.Context) {
	videos = []video{}
	c.IndentedJSON(http.StatusOK, videos)
}

func delvideoByID(c *gin.Context) {
	inputId := c.Param("id")

	for s, a := range videos {
		if strconv.Itoa(a.ID) == inputId {
			videos = remove(videos, s)
			c.IndentedJSON(http.StatusCreated, videos)
			return
		}
	}
	c.IndentedJSON(http.StatusNotFound, gin.H{"message": "video not found"})
}

func updatevideo(c *gin.Context) {
	inputId := c.Param("id")

	var newvideo video

	if err := c.BindJSON(&newvideo); err != nil {
		return
	}

	for s, a := range videos {
		if strconv.Itoa(a.ID) == inputId {
			videos = remove(videos, s)
			videos = append(videos, newvideo)
			c.IndentedJSON(http.StatusCreated, newvideo)
			return
		}
	}
	c.IndentedJSON(http.StatusNotFound, gin.H{"message": "video not found"})
}
